@@ -7,6 +7,7 @@
 #include <stddef.h>
 
 #include <memory>
+#include <optional>
 #include <utility>
 #include <vector>
 
@@ -24,7 +25,6 @@
 #include "base/values.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 
 namespace {
 
@@ -103,17 +103,21 @@ class StatisticsRecorderTest : public testing::TestWithParam<bool> {
     // Note: We can't clear |top_| in the locked block, because the
     // StatisticsRecorder destructor expects that the lock isn't already held.
     {
-      const StatisticsRecorder::SrAutoWriterLock auto_lock(
-          StatisticsRecorder::GetLock());
+      const AutoLock auto_lock(StatisticsRecorder::GetLock());
       statistics_recorder_.reset(StatisticsRecorder::top_);
+      if (statistics_recorder_) {
+        // Prevent releasing ranges in test to avoid dangling pointers in
+        // created histogram objects.
+        statistics_recorder_->ranges_manager_
+            .DoNotReleaseRangesOnDestroyForTesting();
+      }
     }
     statistics_recorder_.reset();
     DCHECK(!HasGlobalRecorder());
   }
 
   bool HasGlobalRecorder() {
-    const StatisticsRecorder::SrAutoReaderLock auto_lock(
-        StatisticsRecorder::GetLock());
+    const AutoLock auto_lock(StatisticsRecorder::GetLock());
     return StatisticsRecorder::top_ != nullptr;
   }
 
@@ -140,7 +144,6 @@ class StatisticsRecorderTest : public testing::TestWithParam<bool> {
   const bool use_persistent_histogram_allocator_;
 
   std::unique_ptr<StatisticsRecorder> statistics_recorder_;
-  std::unique_ptr<GlobalHistogramAllocator> old_global_allocator_;
 
  private:
   LogStateSaver log_state_saver_;
@@ -220,12 +223,13 @@ TEST_P(StatisticsRecorderTest, FindHistogram) {
 
   // Create a new global allocator using the same memory as the old one. Any
   // old one is kept around so the memory doesn't get released.
-  old_global_allocator_ = GlobalHistogramAllocator::ReleaseForTesting();
+  GlobalHistogramAllocator* old_global_allocator =
+      GlobalHistogramAllocator::ReleaseForTesting();
   if (use_persistent_histogram_allocator_) {
     GlobalHistogramAllocator::CreateWithPersistentMemory(
-        const_cast<void*>(old_global_allocator_->data()),
-        old_global_allocator_->length(), 0, old_global_allocator_->Id(),
-        old_global_allocator_->Name());
+        const_cast<void*>(old_global_allocator->data()),
+        old_global_allocator->length(), 0, old_global_allocator->Id(),
+        old_global_allocator->Name());
   }
 
   // Reset statistics-recorder to validate operation from a clean start.
@@ -360,7 +364,7 @@ TEST_P(StatisticsRecorderTest, ToJSON) {
   std::string json(StatisticsRecorder::ToJSON(JSON_VERBOSITY_LEVEL_FULL));
 
   // Check for valid JSON.
-  absl::optional<Value> root = JSONReader::Read(json);
+  std::optional<Value> root = JSONReader::Read(json);
   ASSERT_TRUE(root);
   Value::Dict* root_dict = root->GetIfDict();
   ASSERT_TRUE(root_dict);
@@ -384,22 +388,35 @@ TEST_P(StatisticsRecorderTest, ToJSON) {
   const Value::List* buckets_list = histogram_dict->FindList("buckets");
   ASSERT_TRUE(buckets_list);
   EXPECT_EQ(2u, buckets_list->size());
+}
 
-  // Check the serialized JSON with a different verbosity level.
-  json = StatisticsRecorder::ToJSON(JSON_VERBOSITY_LEVEL_OMIT_BUCKETS);
-  root = JSONReader::Read(json);
+// Check the serialized JSON with a different verbosity level.
+TEST_P(StatisticsRecorderTest, ToJSONOmitBuckets) {
+  Histogram::FactoryGet("TestHistogram1", 1, 1000, 50, HistogramBase::kNoFlags)
+      ->Add(30);
+  Histogram::FactoryGet("TestHistogram1", 1, 1000, 50, HistogramBase::kNoFlags)
+      ->Add(40);
+  Histogram::FactoryGet("TestHistogram2", 1, 1000, 50, HistogramBase::kNoFlags)
+      ->Add(30);
+  Histogram::FactoryGet("TestHistogram2", 1, 1000, 50, HistogramBase::kNoFlags)
+      ->Add(40);
+
+  std::string json =
+      StatisticsRecorder::ToJSON(JSON_VERBOSITY_LEVEL_OMIT_BUCKETS);
+  std::optional<Value> root = JSONReader::Read(json);
   ASSERT_TRUE(root);
-  root_dict = root->GetIfDict();
+  Value::Dict* root_dict = root->GetIfDict();
   ASSERT_TRUE(root_dict);
-  histogram_list = root_dict->FindList("histograms");
+  const Value::List* histogram_list = root_dict->FindList("histograms");
   ASSERT_TRUE(histogram_list);
+
   ASSERT_EQ(2u, histogram_list->size());
   const Value::Dict* histogram_dict2 = (*histogram_list)[0].GetIfDict();
   ASSERT_TRUE(histogram_dict2);
-  sample_count = histogram_dict2->FindInt("count");
+  auto sample_count = histogram_dict2->FindInt("count");
   ASSERT_TRUE(sample_count);
   EXPECT_EQ(2, *sample_count);
-  buckets_list = histogram_dict2->FindList("buckets");
+  const Value::List* buckets_list = histogram_dict2->FindList("buckets");
   // Bucket information should be omitted.
   ASSERT_FALSE(buckets_list);
 }
@@ -415,12 +432,13 @@ TEST_P(StatisticsRecorderTest, IterationTest) {
 
   // Create a new global allocator using the same memory as the old one. Any
   // old one is kept around so the memory doesn't get released.
-  old_global_allocator_ = GlobalHistogramAllocator::ReleaseForTesting();
+  GlobalHistogramAllocator* old_global_allocator =
+      GlobalHistogramAllocator::ReleaseForTesting();
   if (use_persistent_histogram_allocator_) {
     GlobalHistogramAllocator::CreateWithPersistentMemory(
-        const_cast<void*>(old_global_allocator_->data()),
-        old_global_allocator_->length(), 0, old_global_allocator_->Id(),
-        old_global_allocator_->Name());
+        const_cast<void*>(old_global_allocator->data()),
+        old_global_allocator->length(), 0, old_global_allocator->Id(),
+        old_global_allocator->Name());
   }
 
   // Reset statistics-recorder to validate operation from a clean start.
@@ -735,17 +753,9 @@ TEST_P(StatisticsRecorderTest, GlobalCallbackCalled) {
   EXPECT_EQ(callback_callcount, 1u);
 }
 
-#if BUILDFLAG(USE_RUNTIME_VLOG)
-// The following check that StatisticsRecorder::InitLogOnShutdownWhileLocked
-// dumps the histogram graph to vlog if VLOG_IS_ON(1) at runtime. When
-// USE_RUNTIME_VLOG is not set, all vlog levels are determined at build time
-// and default to off. Since we do not want StatisticsRecorder to dump all the
-// time, VLOG in its code stays off. As a result, the following tests would
-// fail.
-
 TEST_P(StatisticsRecorderTest, LogOnShutdownNotInitialized) {
   ResetVLogInitialized();
-  logging::SetMinLogLevel(logging::LOG_WARNING);
+  logging::SetMinLogLevel(logging::LOGGING_WARNING);
   InitializeStatisticsRecorder();
   EXPECT_FALSE(VLOG_IS_ON(1));
   EXPECT_FALSE(IsVLogInitialized());
@@ -755,11 +765,11 @@ TEST_P(StatisticsRecorderTest, LogOnShutdownNotInitialized) {
 
 TEST_P(StatisticsRecorderTest, LogOnShutdownInitializedExplicitly) {
   ResetVLogInitialized();
-  logging::SetMinLogLevel(logging::LOG_WARNING);
+  logging::SetMinLogLevel(logging::LOGGING_WARNING);
   InitializeStatisticsRecorder();
   EXPECT_FALSE(VLOG_IS_ON(1));
   EXPECT_FALSE(IsVLogInitialized());
-  logging::SetMinLogLevel(logging::LOG_VERBOSE);
+  logging::SetMinLogLevel(logging::LOGGING_VERBOSE);
   EXPECT_TRUE(VLOG_IS_ON(1));
   InitLogOnShutdown();
   EXPECT_TRUE(IsVLogInitialized());
@@ -767,17 +777,15 @@ TEST_P(StatisticsRecorderTest, LogOnShutdownInitializedExplicitly) {
 
 TEST_P(StatisticsRecorderTest, LogOnShutdownInitialized) {
   ResetVLogInitialized();
-  logging::SetMinLogLevel(logging::LOG_VERBOSE);
+  logging::SetMinLogLevel(logging::LOGGING_VERBOSE);
   InitializeStatisticsRecorder();
   EXPECT_TRUE(VLOG_IS_ON(1));
   EXPECT_TRUE(IsVLogInitialized());
 }
-#endif  // BUILDFLAG(USE_RUNTIME_VLOG)
 
 class TestHistogramProvider : public StatisticsRecorder::HistogramProvider {
  public:
-  explicit TestHistogramProvider(
-      std::unique_ptr<PersistentHistogramAllocator> allocator)
+  explicit TestHistogramProvider(PersistentHistogramAllocator* allocator)
       : allocator_(std::move(allocator)) {
     StatisticsRecorder::RegisterHistogramProvider(weak_factory_.GetWeakPtr());
   }
@@ -796,7 +804,7 @@ class TestHistogramProvider : public StatisticsRecorder::HistogramProvider {
   }
 
  private:
-  std::unique_ptr<PersistentHistogramAllocator> allocator_;
+  const raw_ptr<PersistentHistogramAllocator> allocator_;
   WeakPtrFactory<TestHistogramProvider> weak_factory_{this};
 };
 
@@ -806,7 +814,7 @@ TEST_P(StatisticsRecorderTest, ImportHistogramsTest) {
       StatisticsRecorder::CreateTemporaryForTesting();
 
   // Extract any existing global allocator so a new one can be created.
-  std::unique_ptr<GlobalHistogramAllocator> old_allocator =
+  GlobalHistogramAllocator* old_allocator =
       GlobalHistogramAllocator::ReleaseForTesting();
 
   // Create a histogram inside a new allocator for testing.
@@ -815,13 +823,13 @@ TEST_P(StatisticsRecorderTest, ImportHistogramsTest) {
   histogram->Add(3);
 
   // Undo back to the starting point.
-  std::unique_ptr<GlobalHistogramAllocator> new_allocator =
+  GlobalHistogramAllocator* new_allocator =
       GlobalHistogramAllocator::ReleaseForTesting();
-  GlobalHistogramAllocator::Set(std::move(old_allocator));
+  GlobalHistogramAllocator::Set(old_allocator);
   temp_sr.reset();
 
   // Create a provider that can supply histograms to the current SR.
-  TestHistogramProvider provider(std::move(new_allocator));
+  TestHistogramProvider provider(new_allocator);
 
   // Verify that the created histogram is no longer known.
   ASSERT_FALSE(StatisticsRecorder::FindHistogram(histogram->histogram_name()));
