@@ -1,12 +1,14 @@
-// Copyright (c) 2013 The Chromium Authors. All rights reserved.
+// Copyright 2013 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/process/process_handle.h"
 
 #include "base/files/file_util.h"
+#include "base/numerics/safe_conversions.h"
 #include "base/process/internal_linux.h"
-#if defined(OS_AIX)
+#include "build/build_config.h"
+#if BUILDFLAG(IS_AIX)
 #include "base/process/internal_aix.h"
 #endif
 
@@ -14,12 +16,14 @@ namespace base {
 
 ProcessId GetParentProcessId(ProcessHandle process) {
   ProcessId pid =
-#if defined(OS_AIX)
+#if BUILDFLAG(IS_AIX)
       internalAIX::ReadProcStatsAndGetFieldAsInt64(process,
                                                    internalAIX::VM_PPID);
 #else
-      internal::ReadProcStatsAndGetFieldAsInt64(process, internal::VM_PPID);
+      checked_cast<ProcessId>(internal::ReadProcStatsAndGetFieldAsInt64(
+          process, internal::VM_PPID));
 #endif
+  // TODO(zijiehe): Returns 0 if |process| does not have a parent process.
   if (pid)
     return pid;
   return -1;
