@@ -1,50 +1,53 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "cc/tiles/checker_image_tracker.h"
 
-#include "base/bind.h"
-#include "base/memory/ptr_util.h"
+#include <algorithm>
+#include <limits>
+#include <sstream>
+#include <string>
+#include <utility>
+
+#include "base/containers/contains.h"
+#include "base/functional/bind.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/stl_util.h"
+#include "base/not_fatal_until.h"
+#include "base/notreached.h"
 #include "base/trace_event/trace_event.h"
 
 namespace cc {
 namespace {
-// The minimum size of an image that we should consider checkering.
-size_t kMinImageSizeToCheckerBytes = 512 * 1024;
-
-// The enum for recording checker-imaging decision UMA metric. Keep this
-// consistent with the ordering in CheckerImagingDecision in enums.xml.
-// Note that this enum is used to back a UMA histogram so should be treated as
-// append only.
 enum class CheckerImagingDecision {
   kCanChecker,
 
   // Animation State vetoes.
   kVetoedAnimatedImage,
   kVetoedVideoFrame,
-  kVetoedAnimationUnknown,
   kVetoedMultipartImage,
 
   // Load state vetoes.
   kVetoedPartiallyLoadedImage,
-  kVetoedLoadStateUnknown,
 
   // Size associated vetoes.
   kVetoedSmallerThanCheckeringSize,
   kVetoedLargerThanCacheSize,
 
-  kCheckerImagingDecisionCount,
+  // Vetoed because checkering of images has been disabled.
+  kVetoedForceDisable,
+
+  // Sync was requested by the embedder.
+  kVetoedSyncRequested,
+
+  kCheckerImagingDecisionCount
 };
 
 std::string ToString(PaintImage::Id paint_image_id,
-                     SkImageId sk_image_id,
                      CheckerImagingDecision decision) {
   std::ostringstream str;
-  str << "paint_image_id[" << paint_image_id << "] sk_image_id[" << sk_image_id
-      << "] decision[" << static_cast<int>(decision) << "]";
+  str << "paint_image_id[" << paint_image_id << "] decision["
+      << static_cast<int>(decision) << "]";
   return str.str();
 }
 
@@ -53,42 +56,46 @@ CheckerImagingDecision GetAnimationDecision(const PaintImage& image) {
     return CheckerImagingDecision::kVetoedMultipartImage;
 
   switch (image.animation_type()) {
-    case PaintImage::AnimationType::UNKNOWN:
-      return CheckerImagingDecision::kVetoedAnimationUnknown;
-    case PaintImage::AnimationType::ANIMATED:
+    case PaintImage::AnimationType::kAnimated:
       return CheckerImagingDecision::kVetoedAnimatedImage;
-    case PaintImage::AnimationType::VIDEO:
+    case PaintImage::AnimationType::kVideo:
       return CheckerImagingDecision::kVetoedVideoFrame;
-    case PaintImage::AnimationType::STATIC:
+    case PaintImage::AnimationType::kStatic:
       return CheckerImagingDecision::kCanChecker;
   }
 
-  NOTREACHED();
+  NOTREACHED_IN_MIGRATION();
   return CheckerImagingDecision::kCanChecker;
 }
 
 CheckerImagingDecision GetLoadDecision(const PaintImage& image) {
   switch (image.completion_state()) {
-    case PaintImage::CompletionState::UNKNOWN:
-      return CheckerImagingDecision::kVetoedLoadStateUnknown;
-    case PaintImage::CompletionState::DONE:
+    case PaintImage::CompletionState::kDone:
       return CheckerImagingDecision::kCanChecker;
-    case PaintImage::CompletionState::PARTIALLY_DONE:
+    case PaintImage::CompletionState::kPartiallyDone:
       return CheckerImagingDecision::kVetoedPartiallyLoadedImage;
   }
 
-  NOTREACHED();
+  NOTREACHED_IN_MIGRATION();
   return CheckerImagingDecision::kCanChecker;
 }
 
-CheckerImagingDecision GetSizeDecision(const PaintImage& image,
+CheckerImagingDecision GetSizeDecision(const SkIRect& src_rect,
+                                       size_t min_bytes,
                                        size_t max_bytes) {
+  // Ideally we would use the original image rect here to estimate the decode
+  // duration for this image. But in the case of sprites/atlases, where small
+  // subsets of this image are used across multiple tiles, re-invalidating for
+  // replacing these images can incur heavy raster cost. So we use the src_rect
+  // here instead.
+  // TODO(khushalsagar): May be we should look at the invalidation rect for an
+  // image here to detect these cases instead?
   base::CheckedNumeric<size_t> checked_size = 4;
-  checked_size *= image.sk_image()->width();
-  checked_size *= image.sk_image()->height();
+  checked_size *= src_rect.width();
+  checked_size *= src_rect.height();
   size_t size = checked_size.ValueOrDefault(std::numeric_limits<size_t>::max());
 
-  if (size < kMinImageSizeToCheckerBytes)
+  if (size < min_bytes)
     return CheckerImagingDecision::kVetoedSmallerThanCheckeringSize;
   else if (size > max_bytes)
     return CheckerImagingDecision::kVetoedLargerThanCacheSize;
@@ -97,6 +104,8 @@ CheckerImagingDecision GetSizeDecision(const PaintImage& image,
 }
 
 CheckerImagingDecision GetCheckerImagingDecision(const PaintImage& image,
+                                                 const SkIRect& src_rect,
+                                                 size_t min_bytes,
                                                  size_t max_bytes) {
   CheckerImagingDecision decision = GetAnimationDecision(image);
   if (decision != CheckerImagingDecision::kCanChecker)
@@ -106,7 +115,7 @@ CheckerImagingDecision GetCheckerImagingDecision(const PaintImage& image,
   if (decision != CheckerImagingDecision::kCanChecker)
     return decision;
 
-  return GetSizeDecision(image, max_bytes);
+  return GetSizeDecision(src_rect, min_bytes, max_bytes);
 }
 
 }  // namespace
@@ -121,11 +130,12 @@ CheckerImageTracker::ImageDecodeRequest::ImageDecodeRequest(
 
 CheckerImageTracker::CheckerImageTracker(ImageController* image_controller,
                                          CheckerImageTrackerClient* client,
-                                         bool enable_checker_imaging)
+                                         bool enable_checker_imaging,
+                                         size_t min_image_bytes_to_checker)
     : image_controller_(image_controller),
       client_(client),
       enable_checker_imaging_(enable_checker_imaging),
-      weak_factory_(this) {}
+      min_image_bytes_to_checker_(min_image_bytes_to_checker) {}
 
 CheckerImageTracker::~CheckerImageTracker() = default;
 
@@ -150,11 +160,6 @@ void CheckerImageTracker::ScheduleImageDecodeQueue(
     ImageDecodeQueue image_decode_queue) {
   TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                "CheckerImageTracker::ScheduleImageDecodeQueue");
-  // Only checker-imaged (async updated) images are decoded using the image
-  // decode service. If |enable_checker_imaging_| is false, no image should
-  // be checkered.
-  DCHECK(image_decode_queue.empty() || enable_checker_imaging_);
-
 #if DCHECK_IS_ON()
   // The decodes in the queue should be prioritized correctly.
   DecodeType type = DecodeType::kRaster;
@@ -204,13 +209,15 @@ void CheckerImageTracker::ClearTracker(bool can_clear_decode_policy_tracking) {
   image_id_to_decode_.clear();
 
   if (can_clear_decode_policy_tracking) {
+    decoding_mode_map_.clear();
     image_async_decode_state_.clear();
+    image_decode_queue_.clear();
   } else {
     // If we can't clear the decode policy, we need to make sure we still
     // re-decode and checker images that were pending invalidation.
     for (auto image_id : images_pending_invalidation_) {
       auto it = image_async_decode_state_.find(image_id);
-      DCHECK(it != image_async_decode_state_.end());
+      CHECK(it != image_async_decode_state_.end(), base::NotFatalUntil::M130);
       DCHECK_EQ(it->second.policy, DecodePolicy::SYNC);
       it->second.policy = DecodePolicy::ASYNC;
     }
@@ -229,8 +236,8 @@ void CheckerImageTracker::DidFinishImageDecode(
     ImageController::ImageDecodeResult result) {
   TRACE_EVENT0(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                "CheckerImageTracker::DidFinishImageDecode");
-  TRACE_EVENT_ASYNC_END0("cc", "CheckerImageTracker::DeferImageDecode",
-                         image_id);
+  TRACE_EVENT_NESTABLE_ASYNC_END0("cc", "CheckerImageTracker::DeferImageDecode",
+                                  TRACE_ID_LOCAL(image_id));
 
   DCHECK_NE(ImageController::ImageDecodeResult::DECODE_NOT_REQUIRED, result);
   DCHECK_EQ(outstanding_image_decode_.value().stable_id(), image_id);
@@ -244,6 +251,18 @@ void CheckerImageTracker::DidFinishImageDecode(
     return;
   }
 
+  // We might have flipped this to sync while updating the hints. That function
+  // would have also requested an invalidation, so we can just schedule the next
+  // decode here.
+  if (it->second.policy == DecodePolicy::SYNC) {
+    // Expensive DCHECK without immediate dereference.
+    DCHECK(decoding_mode_map_.find(image_id) != decoding_mode_map_.end());
+    DCHECK_EQ(decoding_mode_map_[image_id], PaintImage::DecodingMode::kSync);
+
+    ScheduleNextImageDecode();
+    return;
+  }
+
   it->second.policy = DecodePolicy::SYNC;
   images_pending_invalidation_.insert(image_id);
   ScheduleNextImageDecode();
@@ -254,12 +273,16 @@ bool CheckerImageTracker::ShouldCheckerImage(const DrawImage& draw_image,
                                              WhichTree tree) {
   const PaintImage& image = draw_image.paint_image();
   PaintImage::Id image_id = image.stable_id();
-  TRACE_EVENT1("cc", "CheckerImageTracker::ShouldCheckerImage", "image_id",
-               image_id);
+  TRACE_EVENT1("cc.debug", "CheckerImageTracker::ShouldCheckerImage",
+               "image_id", image_id);
 
+  // Checkering of all images is disabled.
   if (!enable_checker_imaging_)
     return false;
 
+  if (!image.IsLazyGenerated())
+    return false;
+
   // If the image was invalidated on the current sync tree and the tile is
   // for the active tree, continue checkering it on the active tree to ensure
   // the image update is atomic for the frame.
@@ -270,11 +293,20 @@ bool CheckerImageTracker::ShouldCheckerImage(const DrawImage& draw_image,
 
   // If the image is pending invalidation, continue checkering it. All tiles
   // for these images will be invalidated on the next pending tree.
-  if (images_pending_invalidation_.find(image_id) !=
-      images_pending_invalidation_.end()) {
+  if (base::Contains(images_pending_invalidation_, image_id)) {
     return true;
   }
 
+  auto decoding_mode_it = decoding_mode_map_.find(image_id);
+  PaintImage::DecodingMode decoding_mode_hint =
+      decoding_mode_it == decoding_mode_map_.end()
+          ? PaintImage::DecodingMode::kUnspecified
+          : decoding_mode_it->second;
+
+  // We only checker images if the developer specifies async decoding mode.
+  if (decoding_mode_hint != PaintImage::DecodingMode::kAsync)
+    return false;
+
   auto insert_result = image_async_decode_state_.insert(
       std::pair<PaintImage::Id, DecodeState>(image_id, DecodeState()));
   auto it = insert_result.first;
@@ -292,19 +324,27 @@ bool CheckerImageTracker::ShouldCheckerImage(const DrawImage& draw_image,
     // 4) Multipart images: Multipart images can be used to display mjpg video
     // frames, checkering which would cause each video frame to flash and
     // therefore should not be checkered.
+    //
+    // Note that we only need to do this check if we didn't veto above in this
+    // block.
     CheckerImagingDecision decision = GetCheckerImagingDecision(
-        image, image_controller_->image_cache_max_limit_bytes());
+        image, draw_image.src_rect(), min_image_bytes_to_checker_,
+        image_controller_->image_cache_max_limit_bytes());
+
+    if (decision == CheckerImagingDecision::kCanChecker && force_disabled_) {
+      // Get the decision for all the veto reasons first, so we can UMA the
+      // images that were not checkered only because checker-imaging was force
+      // disabled.
+      decision = CheckerImagingDecision::kVetoedForceDisable;
+    }
+
     it->second.policy = decision == CheckerImagingDecision::kCanChecker
                             ? DecodePolicy::ASYNC
                             : DecodePolicy::SYNC;
 
-    UMA_HISTOGRAM_ENUMERATION(
-        "Compositing.Renderer.CheckerImagingDecision", decision,
-        CheckerImagingDecision::kCheckerImagingDecisionCount);
-
     TRACE_EVENT1(TRACE_DISABLED_BY_DEFAULT("cc.debug"),
                  "CheckerImageTracker::CheckerImagingDecision", "image_params",
-                 ToString(image_id, image.sk_image()->uniqueID(), decision));
+                 ToString(image_id, decision));
   }
 
   // Update the decode state from the latest image we have seen. Note that it
@@ -336,9 +376,11 @@ void CheckerImageTracker::UpdateDecodeState(const DrawImage& draw_image,
   decode_state->scale = SkSize::Make(
       std::max(decode_state->scale.fWidth, draw_image.scale().fWidth),
       std::max(decode_state->scale.fHeight, draw_image.scale().fHeight));
+  decode_state->use_dark_mode = draw_image.use_dark_mode();
   decode_state->filter_quality =
       std::max(decode_state->filter_quality, draw_image.filter_quality());
-  decode_state->color_space = draw_image.target_color_space();
+  decode_state->target_color_params = draw_image.target_color_params();
+  decode_state->frame_index = draw_image.frame_index();
 }
 
 void CheckerImageTracker::ScheduleNextImageDecode() {
@@ -368,15 +410,16 @@ void CheckerImageTracker::ScheduleNextImageDecode() {
     // needed.
     PaintImage::Id image_id = candidate.stable_id();
     auto it = image_async_decode_state_.find(image_id);
-    DCHECK(it != image_async_decode_state_.end());
+    CHECK(it != image_async_decode_state_.end(), base::NotFatalUntil::M130);
     if (it->second.policy != DecodePolicy::ASYNC)
       continue;
 
-    draw_image = DrawImage(candidate, candidate.sk_image()->bounds(),
-                           it->second.filter_quality,
-                           SkMatrix::MakeScale(it->second.scale.width(),
-                                               it->second.scale.height()),
-                           it->second.color_space);
+    draw_image = DrawImage(
+        candidate, it->second.use_dark_mode,
+        SkIRect::MakeWH(candidate.width(), candidate.height()),
+        it->second.filter_quality,
+        SkM44::Scale(it->second.scale.width(), it->second.scale.height()),
+        it->second.frame_index, it->second.target_color_params);
     outstanding_image_decode_.emplace(candidate);
     break;
   }
@@ -390,15 +433,55 @@ void CheckerImageTracker::ScheduleNextImageDecode() {
 
   PaintImage::Id image_id = outstanding_image_decode_.value().stable_id();
   DCHECK_EQ(image_id_to_decode_.count(image_id), 0u);
-  TRACE_EVENT_ASYNC_BEGIN0("cc", "CheckerImageTracker::DeferImageDecode",
-                           image_id);
+  TRACE_EVENT_NESTABLE_ASYNC_BEGIN0(
+      "cc", "CheckerImageTracker::DeferImageDecode", TRACE_ID_LOCAL(image_id));
   ImageController::ImageDecodeRequestId request_id =
       image_controller_->QueueImageDecode(
-          draw_image, base::Bind(&CheckerImageTracker::DidFinishImageDecode,
-                                 weak_factory_.GetWeakPtr(), image_id));
+          draw_image, base::BindOnce(&CheckerImageTracker::DidFinishImageDecode,
+                                     weak_factory_.GetWeakPtr(), image_id));
 
-  image_id_to_decode_.emplace(image_id, base::MakeUnique<ScopedDecodeHolder>(
+  image_id_to_decode_.emplace(image_id, std::make_unique<ScopedDecodeHolder>(
                                             image_controller_, request_id));
 }
 
+void CheckerImageTracker::UpdateImageDecodingHints(
+    base::flat_map<PaintImage::Id, PaintImage::DecodingMode>
+        decoding_mode_map) {
+  if (!enable_checker_imaging_)
+    return;
+
+  // Merge the |decoding_mode_map| with our member map, keeping the more
+  // conservative values.
+  // TODO(vmpstr): Figure out if and how do we clear this value to ensure that
+  // if we no longer have any kSync images, for example, then we can loosen the
+  // requirement on the decoding mode for that image id.
+  for (auto pair : decoding_mode_map) {
+    PaintImage::Id id = pair.first;
+    PaintImage::DecodingMode decoding_mode = pair.second;
+
+    // In case we already have this image as async, it implies that we are
+    // currently displaying this content as checkered. We can flip the state to
+    // sync here and add the image to be invalidated. The invalidation should
+    // happen shortly after, since this function should be called in a commit.
+    auto state_it = image_async_decode_state_.find(id);
+    if (state_it != image_async_decode_state_.end()) {
+      auto& state = state_it->second;
+      if (state.policy == DecodePolicy::ASYNC &&
+          decoding_mode == PaintImage::DecodingMode::kSync) {
+        state.policy = DecodePolicy::SYNC;
+        images_pending_invalidation_.insert(id);
+      }
+    }
+
+    // Update the decoding hints map.
+    auto decoding_mode_it = decoding_mode_map_.find(id);
+    if (decoding_mode_it == decoding_mode_map_.end()) {
+      decoding_mode_map_[id] = decoding_mode;
+    } else {
+      decoding_mode_it->second =
+          PaintImage::GetConservative(decoding_mode_it->second, decoding_mode);
+    }
+  }
+}
+
 }  // namespace cc
