@@ -1,22 +1,25 @@
-// Copyright 2018 The Chromium Authors. All rights reserved.
+// Copyright 2018 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/net/system_network_context_manager.h"
 
+#include <optional>
 #include <string>
 #include <vector>
 
 #include "base/files/file_path.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_forward.h"
+#include "base/run_loop.h"
 #include "base/strings/strcat.h"
 #include "base/test/bind.h"
 #include "base/test/scoped_feature_list.h"
-#include "base/threading/sequenced_task_runner_handle.h"
+#include "base/test/values_test_util.h"
 #include "base/values.h"
 #include "build/build_config.h"
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/component_updater/first_party_sets_component_installer.h"
 #include "chrome/browser/net/stub_resolver_config_reader.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
@@ -26,20 +29,31 @@
 #include "chrome/common/pref_names.h"
 #include "chrome/test/base/in_process_browser_test.h"
 #include "chrome/test/base/test_launcher_utils.h"
+#include "components/component_updater/installer_policies/first_party_sets_component_installer_policy.h"
 #include "components/prefs/pref_service.h"
 #include "components/version_info/version_info.h"
+#include "content/public/browser/cookie_access_details.h"
+#include "content/public/browser/navigation_handle.h"
 #include "content/public/browser/network_service_instance.h"
+#include "content/public/browser/network_service_util.h"
+#include "content/public/browser/render_frame_host.h"
+#include "content/public/browser/service_process_host.h"
+#include "content/public/browser/service_process_info.h"
+#include "content/public/browser/web_contents_observer.h"
 #include "content/public/common/content_features.h"
-#include "content/public/common/network_service_util.h"
 #include "content/public/common/user_agent.h"
 #include "content/public/test/browser_test.h"
 #include "content/public/test/browser_test_utils.h"
-#include "content/public/test/frame_test_utils.h"
+#include "content/public/test/content_browser_test_utils.h"
 #include "content/public/test/test_utils.h"
+#include "mojo/public/cpp/bindings/sync_call_restrictions.h"
+#include "net/base/features.h"
 #include "net/cookies/canonical_cookie_test_helpers.h"
+#include "net/cookies/cookie_access_result.h"
+#include "net/cookies/cookie_inclusion_status.h"
 #include "net/dns/mock_host_resolver.h"
 #include "net/net_buildflags.h"
-#include "services/cert_verifier/test_cert_verifier_service_factory.h"
+#include "sandbox/policy/features.h"
 #include "services/network/public/cpp/features.h"
 #include "services/network/public/cpp/network_service_buildflags.h"
 #include "services/network/public/mojom/network_context.mojom.h"
@@ -48,20 +62,17 @@
 #include "testing/gmock/include/gmock/gmock-matchers.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "third_party/blink/public/common/features.h"
 
-#if BUILDFLAG(BUILTIN_CERT_VERIFIER_FEATURE_SUPPORTED)
-#include "chrome/browser/policy/policy_test_utils.h"
-#include "components/policy/core/common/policy_map.h"
-#include "components/policy/policy_constants.h"
-#include "net/base/features.h"
-#endif
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_LINUX)
+#include "sandbox/policy/linux/sandbox_seccomp_bpf_linux.h"
+#endif  // BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_LINUX)
 
 using SystemNetworkContextManagerBrowsertest = InProcessBrowserTest;
 
-const char* kSamePartyCookieName = "SamePartyCookie";
+const char* kCookieName = "Cookie";
 const char* kHostA = "a.test";
+const char* kHostB = "b.test";
 
 IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest,
                        StaticAuthParams) {
@@ -69,7 +80,7 @@ IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest,
   network::mojom::HttpAuthStaticParamsPtr static_params =
       SystemNetworkContextManager::GetHttpAuthStaticParamsForTesting();
   EXPECT_EQ("", static_params->gssapi_library_name);
-#if BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_ANDROID) && !BUILDFLAG(IS_CHROMEOS)
   // Test that prefs are reflected in params.
 
   PrefService* local_state = g_browser_process->local_state();
@@ -78,7 +89,8 @@ IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest,
   static_params =
       SystemNetworkContextManager::GetHttpAuthStaticParamsForTesting();
   EXPECT_EQ(dev_null, static_params->gssapi_library_name);
-#endif
+#endif  // BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_ANDROID) &&
+        // !BUILDFLAG(IS_CHROMEOS)
 }
 
 IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest, AuthParams) {
@@ -190,7 +202,7 @@ IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest, AuthParams) {
   EXPECT_TRUE(dynamic_params->patterns_allowed_to_use_all_schemes.empty());
 #endif  // BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_MAC) || BUILDFLAG(IS_CHROMEOS)
 
-#if BUILDFLAG(IS_CHROMEOS_ASH)
+#if BUILDFLAG(IS_CHROMEOS)
   // The kerberos.enabled pref is false and the device is not Active Directory
   // managed by default.
   EXPECT_FALSE(dynamic_params->allow_gssapi_library_load);
@@ -199,14 +211,13 @@ IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest, AuthParams) {
       SystemNetworkContextManager::GetHttpAuthDynamicParamsForTesting();
   EXPECT_TRUE(dynamic_params->allow_gssapi_library_load);
   EXPECT_TRUE(dynamic_params->patterns_allowed_to_use_all_schemes.empty());
-#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+#endif  // BUILDFLAG(IS_CHROMEOS)
 
-  base::Value patterns_allowed_to_use_all_schemes(base::Value::Type::LIST);
-  patterns_allowed_to_use_all_schemes.Append(
-      base::Value("*.allowed.google.com"));
-  patterns_allowed_to_use_all_schemes.Append(base::Value("*.youtube.com"));
-  local_state->Set(prefs::kAllHttpAuthSchemesAllowedForOrigins,
-                   std::move(patterns_allowed_to_use_all_schemes));
+  base::Value::List patterns_allowed_to_use_all_schemes;
+  patterns_allowed_to_use_all_schemes.Append("*.allowed.google.com");
+  patterns_allowed_to_use_all_schemes.Append("*.youtube.com");
+  local_state->SetList(prefs::kAllHttpAuthSchemesAllowedForOrigins,
+                       std::move(patterns_allowed_to_use_all_schemes));
   dynamic_params =
       SystemNetworkContextManager::GetHttpAuthDynamicParamsForTesting();
 
@@ -219,6 +230,321 @@ IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerBrowsertest, AuthParams) {
             dynamic_params->patterns_allowed_to_use_all_schemes);
 }
 
+#if BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_LINUX)
+// GSSAPI is currently incompatible with the network service sandbox
+// (crbug.com/1474362). It isn't known until the browser is already started
+// whether GSSAPI is desired, so if Chrome detects that GSSAPI is desired after
+// the network service has already started sandboxed, the network
+// service must be restarted so the sandbox can be removed.
+class SystemNetworkContextManagerNetworkServiceSandboxBrowsertest
+    : public SystemNetworkContextManagerBrowsertest,
+      public content::ServiceProcessHost::Observer,
+      public testing::WithParamInterface<bool> {
+ public:
+  // On both ChromeOS and Linux, a pref determines whether GSSAPI is desired in
+  // the network service. This pref will determine whether the network service
+  // is sandboxed, and when it changes from false to true this should trigger a
+  // network service restart to remove the sandbox.
+  const char* kGssapiDesiredPref =
+#if BUILDFLAG(IS_CHROMEOS)
+      prefs::kKerberosEnabled;
+#elif BUILDFLAG(IS_LINUX)
+      prefs::kReceivedHttpAuthNegotiateHeader;
+#endif
+
+  SystemNetworkContextManagerNetworkServiceSandboxBrowsertest() {
+    sandbox_desired_ = GetParam();
+    scoped_feature_list_.InitWithFeatureState(
+        sandbox::policy::features::kNetworkServiceSandbox, sandbox_desired_);
+  }
+
+  void SetUpOnMainThread() override {
+    // If the sandbox or the seccomp policy is disabled, these tests are
+    // meaningless.
+    if (!sandbox::policy::SandboxSeccompBPF::IsSeccompBPFDesired()) {
+      GTEST_SKIP();
+    }
+
+    SystemNetworkContextManagerBrowsertest::SetUpOnMainThread();
+
+    content::ServiceProcessHost::AddObserver(this);
+    auto running_processes =
+        content::ServiceProcessHost::GetRunningProcessInfo();
+    for (const auto& info : running_processes) {
+      if (info.IsService<network::mojom::NetworkService>()) {
+        network_process_ = info.GetProcess().Duplicate();
+        break;
+      }
+    }
+  }
+
+  void WaitForNextLaunch() {
+    launch_run_loop_.emplace();
+    launch_run_loop_->Run();
+  }
+
+  void ExpectNetworkService(bool seccomp_sandboxed,
+                            bool allows_gssapi_library_load) {
+    // The network service may have been launched but has not yet sandboxed
+    // itself. So, wait for the Mojo endpoints to start accepting messages.
+    mojo::Remote<network::mojom::NetworkServiceTest> network_service_test;
+    content::GetNetworkService()->BindTestInterfaceForTesting(
+        network_service_test.BindNewPipeAndPassReceiver());
+    mojo::ScopedAllowSyncCallForTesting allow_sync_call;
+    // Log() is sync so this thread will wait for this call to succeed.
+    network_service_test->Log(
+        "Logging in network service to ensure it's ready.");
+
+    // Now the test can check if the seccomp sandbox has been applied.
+    EXPECT_EQ(seccomp_sandboxed,
+              GetNetworkServiceProcess().IsSeccompSandboxed());
+
+    bool network_service_allows_gssapi_library_load;
+    ASSERT_TRUE(network_service_test->AllowsGSSAPILibraryLoad(
+        &network_service_allows_gssapi_library_load));
+    EXPECT_EQ(allows_gssapi_library_load,
+              network_service_allows_gssapi_library_load);
+  }
+
+  base::Process GetNetworkServiceProcess() {
+    CHECK(content::IsOutOfProcessNetworkService());
+    return network_process_.Duplicate();
+  }
+
+ protected:
+  bool sandbox_desired_;
+
+ private:
+  void OnServiceProcessLaunched(
+      const content::ServiceProcessInfo& info) override {
+    if (!info.IsService<network::mojom::NetworkService>()) {
+      return;
+    }
+    network_process_ = info.GetProcess().Duplicate();
+    if (launch_run_loop_) {
+      launch_run_loop_->Quit();
+    }
+  }
+
+  void OnServiceProcessTerminatedNormally(
+      const content::ServiceProcessInfo& info) override {}
+
+  void OnServiceProcessCrashed(
+      const content::ServiceProcessInfo& info) override {}
+
+  base::test::ScopedFeatureList scoped_feature_list_;
+  base::Process network_process_;
+  std::optional<base::RunLoop> launch_run_loop_;
+};
+
+IN_PROC_BROWSER_TEST_P(
+    SystemNetworkContextManagerNetworkServiceSandboxBrowsertest,
+    NetworkServiceRestartsUnsandboxedOnGssapiDesired) {
+  PrefService* local_state = g_browser_process->local_state();
+
+  // Ensure GSSAPI starts as "undesired".
+  EXPECT_FALSE(local_state->GetBoolean(kGssapiDesiredPref));
+  // Ensure the network service starts sandboxed (if desired) and cannot load
+  // GSSAPI libraries.
+  ExpectNetworkService(/*seccomp_sandboxed=*/sandbox_desired_,
+                       /*allows_gssapi_library_load=*/false);
+
+  // Now signal that GSSAPI is desired.
+  local_state->SetBoolean(kGssapiDesiredPref, true);
+  EXPECT_TRUE(local_state->GetBoolean(kGssapiDesiredPref));
+  // If the network service was sandboxed it should automatically restart and
+  // be unsandboxed. In any case it should now respect the pref and allow GSSAPI
+  // library loads.
+  if (sandbox_desired_) {
+    WaitForNextLaunch();
+  }
+  ExpectNetworkService(/*seccomp_sandboxed=*/false,
+                       /*allows_gssapi_library_load=*/true);
+
+  // After killing the network service, it should still restart unsandboxed and
+  // allow GSSAPI library loads.
+  base::SequencedTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE,
+      base::BindOnce(base::IgnoreResult(&content::RestartNetworkService)));
+  WaitForNextLaunch();
+  ExpectNetworkService(/*seccomp_sandboxed=*/false,
+                       /*allows_gssapi_library_load=*/true);
+}
+
+IN_PROC_BROWSER_TEST_P(
+    SystemNetworkContextManagerNetworkServiceSandboxBrowsertest,
+    PRE_NetworkServiceStartsUnsandboxedWithGssapiDesired) {
+  PrefService* local_state = g_browser_process->local_state();
+  // Signal that GSSAPI is desired. This should persist across browser restarts
+  // like any pref.
+  local_state->SetBoolean(kGssapiDesiredPref, true);
+  EXPECT_TRUE(local_state->GetBoolean(kGssapiDesiredPref));
+}
+
+IN_PROC_BROWSER_TEST_P(
+    SystemNetworkContextManagerNetworkServiceSandboxBrowsertest,
+    NetworkServiceStartsUnsandboxedWithGssapiDesired) {
+  // Ensure the network service starts sandboxed and allows GSSAPI library
+  // loads.
+  ExpectNetworkService(/*seccomp_sandboxed=*/false,
+                       /*allows_gssapi_library_load=*/true);
+}
+
+INSTANTIATE_TEST_SUITE_P(
+    All,
+    SystemNetworkContextManagerNetworkServiceSandboxBrowsertest,
+    testing::Bool(),
+    [](const testing::TestParamInfo<bool>& info) {
+      return info.param ? "NetworkSandboxDesired"
+                        : "NetworkSandboxFullyDisabled";
+    });
+
+#endif  // BUILDFLAG(IS_CHROMEOS) || BUILDFLAG(IS_LINUX)
+
+#if BUILDFLAG(IS_LINUX)
+class SystemNetworkContextManagerHttpNegotiateHeader
+    : public SystemNetworkContextManagerBrowsertest {
+ public:
+  static constexpr char kHttpsNegotiateAuthPath[] = "/http_negotiate_auth";
+
+  SystemNetworkContextManagerHttpNegotiateHeader()
+      : https_server_(net::EmbeddedTestServer::TYPE_HTTPS) {}
+
+  void SetUpOnMainThread() override {
+    SystemNetworkContextManagerBrowsertest::SetUpOnMainThread();
+
+    https_server_.AddDefaultHandlers(
+        base::FilePath(FILE_PATH_LITERAL("content/test/data")));
+    https_server_.RegisterRequestHandler(
+        base::BindRepeating(&SystemNetworkContextManagerHttpNegotiateHeader::
+                                SendBackHttpNegotiateHeader,
+                            base::Unretained(this)));
+    ASSERT_TRUE(https_server_.Start());
+  }
+
+  std::unique_ptr<net::test_server::HttpResponse> SendBackHttpNegotiateHeader(
+      const net::test_server::HttpRequest& request) {
+    if (request.relative_url != kHttpsNegotiateAuthPath) {
+      return nullptr;
+    }
+
+    auto http_response =
+        std::make_unique<net::test_server::BasicHttpResponse>();
+    http_response->set_code(net::HTTP_UNAUTHORIZED);
+    http_response->AddCustomHeader("WWW-Authenticate", "Negotiate");
+    return http_response;
+  }
+
+  content::WebContents* web_contents() {
+    return browser()->tab_strip_model()->GetActiveWebContents();
+  }
+
+ protected:
+  net::test_server::EmbeddedTestServer https_server_;
+};
+
+IN_PROC_BROWSER_TEST_F(SystemNetworkContextManagerHttpNegotiateHeader,
+                       DISABLED_SetsPrefOnHttpNegotiateHeader) {
+  PrefService* local_state = g_browser_process->local_state();
+
+  // Ensure the pref starts false.
+  EXPECT_FALSE(
+      local_state->GetBoolean(prefs::kReceivedHttpAuthNegotiateHeader));
+
+  PrefChangeRegistrar pref_change_registrar;
+  pref_change_registrar.Init(local_state);
+
+  base::RunLoop wait_for_set_pref_loop;
+  pref_change_registrar.Add(prefs::kReceivedHttpAuthNegotiateHeader,
+                            wait_for_set_pref_loop.QuitClosure());
+
+  // Navigate to a URL that requests negotiate authentication.
+  EXPECT_FALSE(NavigateToURL(web_contents(),
+                             https_server_.GetURL(kHttpsNegotiateAuthPath)));
+  wait_for_set_pref_loop.Run();
+
+  // Ensure the pref is now true.
+  EXPECT_TRUE(local_state->GetBoolean(prefs::kReceivedHttpAuthNegotiateHeader));
+}
+#endif  // BUILDFLAG(IS_LINUX)
+
+namespace {
+struct CookieAccess {
+  content::CookieAccessDetails::Type type;
+  std::string cookie_name;
+  std::string cookie_value;
+  net::CookieAccessResult cookie_access_result;
+
+  friend std::ostream& operator<<(std::ostream& o, const CookieAccess& d) {
+    o << (d.type == content::CookieAccessDetails::Type::kRead ? "read"
+                                                              : "change");
+    o << " name=" << d.cookie_name;
+    o << " value=" << d.cookie_value;
+    o << " access_result=";
+    net::PrintTo(d.cookie_access_result, &o);
+    return o;
+  }
+
+ public:
+  bool operator==(const CookieAccess&) const = default;
+};
+
+class CookieTracker : public content::WebContentsObserver {
+ public:
+  explicit CookieTracker(content::WebContents* web_contents)
+      : content::WebContentsObserver(web_contents) {}
+
+  void OnCookiesAccessed(content::NavigationHandle* navigation,
+                         const content::CookieAccessDetails& details) override {
+    OnCookiesAccessed(details);
+  }
+
+  void OnCookiesAccessed(content::RenderFrameHost* rfh,
+                         const content::CookieAccessDetails& details) override {
+    OnCookiesAccessed(details);
+  }
+
+  void WaitForCookies(size_t count) {
+    waiting_for_cookies_count_ = count;
+
+    base::RunLoop run_loop;
+    quit_closure_ = run_loop.QuitClosure();
+    QuitIfReady();
+    run_loop.Run();
+  }
+
+  std::vector<CookieAccess>& cookie_accesses() { return cookie_accesses_; }
+
+ private:
+  void OnCookiesAccessed(const content::CookieAccessDetails& details) {
+    for (const auto& cookie_with_access_result :
+         details.cookie_access_result_list) {
+      cookie_accesses_.emplace_back(details.type,
+                                    cookie_with_access_result.cookie.Name(),
+                                    cookie_with_access_result.cookie.Value(),
+                                    cookie_with_access_result.access_result);
+    }
+
+    QuitIfReady();
+  }
+
+  void QuitIfReady() {
+    if (quit_closure_.is_null()) {
+      return;
+    }
+    if (cookie_accesses_.size() < waiting_for_cookies_count_) {
+      return;
+    }
+    std::move(quit_closure_).Run();
+  }
+
+  std::vector<CookieAccess> cookie_accesses_;
+  size_t waiting_for_cookies_count_ = 0;
+  base::OnceClosure quit_closure_;
+};
+
+}  // namespace
+
 class SystemNetworkContextManagerWithFirstPartySetComponentBrowserTest
     : public SystemNetworkContextManagerBrowsertest {
  public:
@@ -236,22 +562,26 @@ class SystemNetworkContextManagerWithFirstPartySetComponentBrowserTest
 
   void SetUpInProcessBrowserTestFixture() override {
     SystemNetworkContextManagerBrowsertest::SetUpInProcessBrowserTestFixture();
-    feature_list_.InitAndEnableFeature(features::kFirstPartySets);
+    // Since we set kWaitForFirstPartySetsInit, all cookie-carrying network
+    // requests are blocked until FPS is initialized.
+    feature_list_.InitWithFeatures(
+        {net::features::kWaitForFirstPartySetsInit,
+         net::features::kForceThirdPartyCookieBlocking},
+        {});
     CHECK(component_dir_.CreateUniqueTempDir());
     base::ScopedAllowBlockingForTesting allow_blocking;
 
     component_updater::FirstPartySetsComponentInstallerPolicy::
-        WriteComponentForTesting(component_dir_.GetPath(),
+        WriteComponentForTesting(base::Version("1.2.3"),
+                                 component_dir_.GetPath(),
                                  GetComponentContents());
   }
 
  protected:
   void SetUpDefaultCommandLine(base::CommandLine* command_line) override {
-    base::CommandLine default_command_line(base::CommandLine::NO_PROGRAM);
     SystemNetworkContextManagerBrowsertest::SetUpDefaultCommandLine(
-        &default_command_line);
-    test_launcher_utils::RemoveCommandLineSwitch(
-        default_command_line, switches::kDisableComponentUpdate, command_line);
+        command_line);
+    command_line->RemoveSwitch(switches::kDisableComponentUpdate);
   }
 
   net::test_server::EmbeddedTestServer* https_server() {
@@ -262,16 +592,12 @@ class SystemNetworkContextManagerWithFirstPartySetComponentBrowserTest
     return browser()->tab_strip_model()->GetActiveWebContents();
   }
 
-  GURL EchoCookiesUrl(const std::string& host) {
-    return https_server_.GetURL(host, "/echoheader?Cookie");
-  }
-
  private:
   std::string GetComponentContents() const {
-    return "{\"owner\": \"https://a.test\", \"members\": [ "
-           "\"https://b.test\", \"https://member1.test\"]}\n"
-           "{\"owner\": \"https://c.test\", \"members\": [ "
-           "\"https://d.test\", \"https://member2.test\"]}";
+    return "{\"primary\": \"https://a.test\", \"associatedSites\": [ "
+           "\"https://b.test\", \"https://associatedsite1.test\"]}\n"
+           "{\"primary\": \"https://c.test\", \"associatedSites\": [ "
+           "\"https://d.test\", \"https://associatedsite2.test\"]}";
   }
 
   base::test::ScopedFeatureList feature_list_;
@@ -295,11 +621,10 @@ IN_PROC_BROWSER_TEST_F(
   ASSERT_TRUE(content::SetCookie(
       browser()->profile(), host_root,
       base::StrCat(
-          {kSamePartyCookieName,
-           "=1; samesite=lax; secure; sameparty; max-age=2147483647"})));
+          {kCookieName, "=1; SameSite=None; secure; max-age=2147483647"})));
   ASSERT_THAT(content::GetCookies(browser()->profile(), host_root),
-              net::CookieStringIs(testing::UnorderedElementsAre(
-                  testing::Key(kSamePartyCookieName))));
+              net::CookieStringIs(
+                  testing::UnorderedElementsAre(testing::Key(kCookieName))));
 }
 
 IN_PROC_BROWSER_TEST_F(
@@ -309,24 +634,64 @@ IN_PROC_BROWSER_TEST_F(
   if (!content::IsOutOfProcessNetworkService())
     return;
 
-  const GURL host_root = https_server()->GetURL(kHostA, "/");
-  ASSERT_THAT(content::GetCookies(browser()->profile(), host_root),
-              net::CookieStringIs(testing::UnorderedElementsAre(
-                  testing::Key(kSamePartyCookieName))));
+  CookieTracker cookie_tracker(web_contents());
 
-  EXPECT_THAT(content::ArrangeFramesAndGetContentFromLeaf(
-                  web_contents(), https_server(), "b.test(%s)", {0},
-                  EchoCookiesUrl(kHostA)),
-              net::CookieStringIs(testing::UnorderedElementsAre(
-                  testing::Key(kSamePartyCookieName))));
+  const GURL url_a = https_server()->GetURL(kHostA, "/title1.html");
+  ASSERT_TRUE(content::NavigateToURL(web_contents(), url_a));
+  cookie_tracker.WaitForCookies(2);
+  const CookieAccess expected_first_party_access{
+      content::CookieAccessDetails::Type::kRead, "Cookie", "1",
+      net::CookieAccessResult(
+          net::CookieEffectiveSameSite::NO_RESTRICTION,
+          net::CookieInclusionStatus(
+              net::CookieInclusionStatus::WARN_PORT_MISMATCH),
+          net::CookieAccessSemantics::NONLEGACY, true)};
+  EXPECT_THAT(cookie_tracker.cookie_accesses(),
+              testing::ElementsAre(
+                  // a.test/title1.html
+                  expected_first_party_access,
+                  // a.test/favicon.ico
+                  expected_first_party_access));
+  cookie_tracker.cookie_accesses().clear();
+
+  const GURL url_b_cross_site(https_server()->GetURL(
+      kHostB, "/cross_site_iframe_factory.html?b.test(a.test)"));
+  EXPECT_TRUE(NavigateToURL(web_contents(), url_b_cross_site));
+  cookie_tracker.WaitForCookies(2);
+  net::CookieInclusionStatus expected_third_party_inclusion_status;
+  // If the sites are in the same Related Website Sets, we're expecting the
+  // EXCLUDE_THIRD_PARTY_BLOCKED_WITHIN_FIRST_PARTY_SET exclusion reason.
+  expected_third_party_inclusion_status.AddExclusionReason(
+      net::CookieInclusionStatus::
+          EXCLUDE_THIRD_PARTY_BLOCKED_WITHIN_FIRST_PARTY_SET);
+  expected_third_party_inclusion_status.AddExclusionReason(
+      net::CookieInclusionStatus::EXCLUDE_THIRD_PARTY_PHASEOUT);
+  expected_third_party_inclusion_status.AddWarningReason(
+      net::CookieInclusionStatus::WARN_PORT_MISMATCH);
+  const CookieAccess expected_third_party_access{
+      content::CookieAccessDetails::Type::kRead, "Cookie", "1",
+      net::CookieAccessResult(net::CookieEffectiveSameSite::NO_RESTRICTION,
+                              expected_third_party_inclusion_status,
+                              net::CookieAccessSemantics::NONLEGACY, true)};
+  EXPECT_THAT(cookie_tracker.cookie_accesses(),
+              testing::ElementsAre(
+                  // a.test iframe under b.test
+                  expected_third_party_access,
+                  // a.test/tree_parser_util.js in an iframe under b.test
+                  expected_third_party_access));
+  cookie_tracker.cookie_accesses().clear();
 
   SimulateNetworkServiceCrash();
 
-  EXPECT_THAT(content::ArrangeFramesAndGetContentFromLeaf(
-                  web_contents(), https_server(), "b.test(%s)", {0},
-                  EchoCookiesUrl(kHostA)),
-              net::CookieStringIs(testing::UnorderedElementsAre(
-                  testing::Key(kSamePartyCookieName))));
+  EXPECT_TRUE(NavigateToURL(web_contents(), url_b_cross_site));
+  cookie_tracker.WaitForCookies(2);
+  EXPECT_THAT(cookie_tracker.cookie_accesses(),
+              testing::ElementsAre(
+                  // a.test iframe under b.test
+                  expected_third_party_access,
+                  // a.test/tree_parser_util.js in an iframe under b.test
+                  expected_third_party_access));
+  cookie_tracker.cookie_accesses().clear();
 }
 
 class SystemNetworkContextManagerReferrersFeatureBrowsertest
@@ -337,7 +702,7 @@ class SystemNetworkContextManagerReferrersFeatureBrowsertest
     scoped_feature_list_.InitWithFeatureState(features::kNoReferrers,
                                               GetParam());
   }
-  ~SystemNetworkContextManagerReferrersFeatureBrowsertest() override {}
+  ~SystemNetworkContextManagerReferrersFeatureBrowsertest() override = default;
 
   void SetUpOnMainThread() override {}
 
@@ -360,14 +725,10 @@ INSTANTIATE_TEST_SUITE_P(All,
                          ::testing::Bool());
 
 class SystemNetworkContextManagerFreezeQUICUaBrowsertest
-    : public SystemNetworkContextManagerBrowsertest,
-      public testing::WithParamInterface<bool> {
+    : public SystemNetworkContextManagerBrowsertest {
  public:
-  SystemNetworkContextManagerFreezeQUICUaBrowsertest() {
-    scoped_feature_list_.InitWithFeatureState(blink::features::kReduceUserAgent,
-                                              GetParam());
-  }
-  ~SystemNetworkContextManagerFreezeQUICUaBrowsertest() override {}
+  SystemNetworkContextManagerFreezeQUICUaBrowsertest() = default;
+  ~SystemNetworkContextManagerFreezeQUICUaBrowsertest() override = default;
 
   void SetUpOnMainThread() override {}
 
@@ -375,32 +736,6 @@ class SystemNetworkContextManagerFreezeQUICUaBrowsertest
   base::test::ScopedFeatureList scoped_feature_list_;
 };
 
-IN_PROC_BROWSER_TEST_P(SystemNetworkContextManagerFreezeQUICUaBrowsertest,
-                       QUICUaConfig) {
-  network::mojom::NetworkContextParamsPtr network_context_params =
-      g_browser_process->system_network_context_manager()
-          ->CreateDefaultNetworkContextParams();
-
-  std::string quic_ua = network_context_params->quic_user_agent_id;
-
-  if (GetParam()) {  // if the UA Freeze feature is turned on
-    EXPECT_EQ("", quic_ua);
-  } else {
-    EXPECT_THAT(quic_ua, testing::HasSubstr(chrome::GetChannelName(
-                             chrome::WithExtendedStable(false))));
-    EXPECT_THAT(quic_ua,
-                testing::HasSubstr(
-                    version_info::GetProductNameAndVersionForUserAgent()));
-    EXPECT_THAT(quic_ua, testing::HasSubstr(content::BuildOSCpuInfo(
-                             content::IncludeAndroidBuildNumber::Exclude,
-                             content::IncludeAndroidModel::Include)));
-  }
-}
-
-INSTANTIATE_TEST_SUITE_P(All,
-                         SystemNetworkContextManagerFreezeQUICUaBrowsertest,
-                         ::testing::Bool());
-
 class SystemNetworkContextManagerWPADQuickCheckBrowsertest
     : public SystemNetworkContextManagerBrowsertest,
       public testing::WithParamInterface<bool> {
@@ -426,7 +761,7 @@ INSTANTIATE_TEST_SUITE_P(All,
 
 class SystemNetworkContextManagerCertificateTransparencyBrowsertest
     : public SystemNetworkContextManagerBrowsertest,
-      public testing::WithParamInterface<absl::optional<bool>> {
+      public testing::WithParamInterface<std::optional<bool>> {
  public:
   SystemNetworkContextManagerCertificateTransparencyBrowsertest() {
     SystemNetworkContextManager::SetEnableCertificateTransparencyForTesting(
@@ -434,120 +769,6 @@ class SystemNetworkContextManagerCertificateTransparencyBrowsertest
   }
   ~SystemNetworkContextManagerCertificateTransparencyBrowsertest() override {
     SystemNetworkContextManager::SetEnableCertificateTransparencyForTesting(
-        absl::nullopt);
+        std::nullopt);
   }
 };
-
-#if BUILDFLAG(BUILTIN_CERT_VERIFIER_FEATURE_SUPPORTED)
-class SystemNetworkContextServiceCertVerifierBuiltinPermissionsPolicyTest
-    : public policy::PolicyTest,
-      public testing::WithParamInterface<bool> {
- public:
-  SystemNetworkContextServiceCertVerifierBuiltinPermissionsPolicyTest() {
-    bool use_builtin_cert_verifier = GetParam();
-    cert_verifier_impl_ =
-        use_builtin_cert_verifier
-            ? cert_verifier::mojom::CertVerifierCreationParams::
-                  CertVerifierImpl::kBuiltin
-            : cert_verifier::mojom::CertVerifierCreationParams::
-                  CertVerifierImpl::kSystem;
-  }
-
-  void SetUpInProcessBrowserTestFixture() override {
-    scoped_feature_list_.InitWithFeatureState(
-        net::features::kCertVerifierBuiltinFeature,
-        cert_verifier_impl_ ==
-            cert_verifier::mojom::CertVerifierCreationParams::CertVerifierImpl::
-                kBuiltin);
-
-    content::SetCertVerifierServiceFactoryForTesting(
-        &test_cert_verifier_service_factory_);
-
-    policy::PolicyTest::SetUpInProcessBrowserTestFixture();
-  }
-
-  void TearDownInProcessBrowserTestFixture() override {
-    content::SetCertVerifierServiceFactoryForTesting(nullptr);
-  }
-
-  void SetUpOnMainThread() override {
-    test_cert_verifier_service_factory_.ReleaseAllCertVerifierParams();
-  }
-
-  void ExpectUseBuiltinCertVerifierCorrect(
-      network::mojom::NetworkContextParamsPtr& network_context_params_ptr,
-      cert_verifier::mojom::CertVerifierCreationParams::CertVerifierImpl
-          use_builtin_cert_verifier) {
-    ASSERT_TRUE(network_context_params_ptr);
-    ASSERT_TRUE(network_context_params_ptr->cert_verifier_params);
-    ASSERT_EQ(1ul, test_cert_verifier_service_factory_.num_captured_params());
-    ASSERT_TRUE(test_cert_verifier_service_factory_.GetParamsAtIndex(0)
-                    ->creation_params);
-    EXPECT_EQ(use_builtin_cert_verifier,
-              test_cert_verifier_service_factory_.GetParamsAtIndex(0)
-                  ->creation_params->use_builtin_cert_verifier);
-    // Send it to the actual CertVerifierServiceFactory.
-    test_cert_verifier_service_factory_.ReleaseNextCertVerifierParams();
-  }
-
-  cert_verifier::mojom::CertVerifierCreationParams::CertVerifierImpl
-  cert_verifier_impl() const {
-    return cert_verifier_impl_;
-  }
-
- private:
-  cert_verifier::mojom::CertVerifierCreationParams::CertVerifierImpl
-      cert_verifier_impl_;
-  base::test::ScopedFeatureList scoped_feature_list_;
-
-  cert_verifier::TestCertVerifierServiceFactoryImpl
-      test_cert_verifier_service_factory_;
-};
-
-IN_PROC_BROWSER_TEST_P(
-    SystemNetworkContextServiceCertVerifierBuiltinPermissionsPolicyTest,
-    Test) {
-  network::mojom::NetworkContextParamsPtr network_context_params_ptr;
-
-  // If no BuiltinCertificateVerifierEnabled policy is set, the
-  // use_builtin_cert_verifier param should be set from the feature flag.
-  network_context_params_ptr =
-      g_browser_process->system_network_context_manager()
-          ->CreateDefaultNetworkContextParams();
-  ExpectUseBuiltinCertVerifierCorrect(network_context_params_ptr,
-                                      cert_verifier_impl());
-#if BUILDFLAG(BUILTIN_CERT_VERIFIER_POLICY_SUPPORTED)
-  // If the BuiltinCertificateVerifierEnabled policy is set it should
-  // override the feature flag.
-  policy::PolicyMap policies;
-  SetPolicy(&policies, policy::key::kBuiltinCertificateVerifierEnabled,
-            base::Value(true));
-  UpdateProviderPolicy(policies);
-
-  network_context_params_ptr =
-      g_browser_process->system_network_context_manager()
-          ->CreateDefaultNetworkContextParams();
-  ExpectUseBuiltinCertVerifierCorrect(
-      network_context_params_ptr,
-      cert_verifier::mojom::CertVerifierCreationParams::CertVerifierImpl::
-          kBuiltin);
-
-  SetPolicy(&policies, policy::key::kBuiltinCertificateVerifierEnabled,
-            base::Value(false));
-  UpdateProviderPolicy(policies);
-
-  network_context_params_ptr =
-      g_browser_process->system_network_context_manager()
-          ->CreateDefaultNetworkContextParams();
-  ExpectUseBuiltinCertVerifierCorrect(
-      network_context_params_ptr,
-      cert_verifier::mojom::CertVerifierCreationParams::CertVerifierImpl::
-          kSystem);
-#endif  // BUILDFLAG(BUILTIN_CERT_VERIFIER_POLICY_SUPPORTED)
-}
-
-INSTANTIATE_TEST_SUITE_P(
-    All,
-    SystemNetworkContextServiceCertVerifierBuiltinPermissionsPolicyTest,
-    ::testing::Bool());
-#endif  // BUILDFLAG(BUILTIN_CERT_VERIFIER_FEATURE_SUPPORTED)
