@@ -17,12 +17,10 @@
 #include "build/chromeos_buildflags.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/google/google_brand.h"
+#include "chrome/browser/lifetime/application_lifetime.h"
 #include "chrome/browser/net/system_network_context_manager.h"
 #include "chrome/browser/prefs/chrome_pref_service_factory.h"
-#include "chrome/browser/profile_resetter/brandcode_config_fetcher.h"
-#include "chrome/browser/profile_resetter/brandcoded_default_settings.h"
 #include "chrome/browser/profile_resetter/profile_resetter.h"
-#include "chrome/browser/profile_resetter/resettable_settings_snapshot.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/grit/generated_resources.h"
@@ -33,8 +31,13 @@
 #include "ui/base/l10n/l10n_util.h"
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-#include "chrome/browser/ash/reset/metrics.h"
-#include "chrome/common/pref_names.h"
+#include "ash/constants/ash_features.h"
+#include "chrome/browser/profiles/profile_manager.h"
+#include "chrome/browser/ui/ash/system_web_apps/system_web_app_ui_utils.h"
+#include "chrome/browser/ui/webui/ash/settings/pref_names.h"
+#include "chromeos/ash/components/network/managed_network_configuration_handler.h"
+#include "chromeos/ash/components/network/network_state_handler.h"
+#include "components/services/app_service/public/cpp/app_launch_util.h"
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
 #if BUILDFLAG(IS_WIN)
@@ -51,8 +54,6 @@ ResetRequestOriginFromString(const std::string& request_origin) {
   static const char kOriginUserClick[] = "userclick";
   static const char kOriginTriggeredReset[] = "triggeredreset";
 
-  if (request_origin == ResetSettingsHandler::kCctResetSettingsHash)
-    return reset_report::ChromeResetReport::RESET_REQUEST_ORIGIN_CCT;
   if (request_origin == kOriginUserClick)
     return reset_report::ChromeResetReport::RESET_REQUEST_ORIGIN_USER_CLICK;
   if (request_origin == kOriginTriggeredReset) {
@@ -60,16 +61,24 @@ ResetRequestOriginFromString(const std::string& request_origin) {
         RESET_REQUEST_ORIGIN_TRIGGERED_RESET;
   }
   if (!request_origin.empty())
-    NOTREACHED();
+    NOTREACHED_IN_MIGRATION();
 
   return reset_report::ChromeResetReport::RESET_REQUEST_ORIGIN_UNKNOWN;
 }
 
 }  // namespace
 
+#if BUILDFLAG(IS_CHROMEOS_ASH)
 // static
 const char ResetSettingsHandler::kCctResetSettingsHash[] = "cct";
 
+// static
+void ResetSettingsHandler::RegisterProfilePrefs(PrefRegistrySimple* registry) {
+  registry->RegisterBooleanPref(ash::settings::prefs::kSanitizeCompleted,
+                                false);
+}
+#endif  // BUILDFLAG(IS_CHROMEOS_ASH)
+
 // static
 bool ResetSettingsHandler::ShouldShowResetProfileBanner(Profile* profile) {
   const base::Time reset_time = chrome_prefs::GetResetTime(profile);
@@ -86,9 +95,8 @@ bool ResetSettingsHandler::ShouldShowResetProfileBanner(Profile* profile) {
 }
 
 ResetSettingsHandler::ResetSettingsHandler(Profile* profile)
-    : profile_(profile) {
-  google_brand::GetBrand(&brandcode_);
-}
+    : profile_(profile),
+      resetter_(std::make_unique<ProfileResetter>(profile_)) {}
 
 ResetSettingsHandler::~ResetSettingsHandler() {}
 
@@ -124,8 +132,12 @@ void ResetSettingsHandler::RegisterMessages() {
           base::Unretained(this)));
 #if BUILDFLAG(IS_CHROMEOS_ASH)
   web_ui()->RegisterMessageCallback(
-      "onPowerwashDialogShow",
-      base::BindRepeating(&ResetSettingsHandler::OnShowPowerwashDialog,
+      "performSanitizeSettings",
+      base::BindRepeating(&ResetSettingsHandler::SanitizeSettings,
+                          base::Unretained(this)));
+  web_ui()->RegisterMessageCallback(
+      "onShowSanitizeDialog",
+      base::BindRepeating(&ResetSettingsHandler::OnShowSanitizeDialog,
                           base::Unretained(this)));
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 }
@@ -141,15 +153,7 @@ void ResetSettingsHandler::HandleResetProfileSettings(
   reset_report::ChromeResetReport::ResetRequestOrigin request_origin =
       ResetRequestOriginFromString(request_origin_string);
 
-  DCHECK(brandcode_.empty() || config_fetcher_);
-  if (config_fetcher_ && config_fetcher_->IsActive()) {
-    // Reset once the prefs are fetched.
-    config_fetcher_->SetCallback(base::BindOnce(
-        &ResetSettingsHandler::ResetProfile, base::Unretained(this),
-        callback_id, send_settings, request_origin));
-  } else {
-    ResetProfile(callback_id, send_settings, request_origin);
-  }
+  ResetProfile(callback_id, send_settings, request_origin);
 }
 
 void ResetSettingsHandler::OnResetProfileSettingsDone(
@@ -196,15 +200,6 @@ void ResetSettingsHandler::OnShowResetProfileDialog(
   if (!GetResetter()->IsActive()) {
     setting_snapshot_ = std::make_unique<ResettableSettingsSnapshot>(profile_);
   }
-
-  if (brandcode_.empty())
-    return;
-  config_fetcher_ = std::make_unique<BrandcodeConfigFetcher>(
-      g_browser_process->system_network_context_manager()
-          ->GetURLLoaderFactory(),
-      base::BindOnce(&ResetSettingsHandler::OnSettingsFetched,
-                     base::Unretained(this)),
-      GURL("https://tools.google.com/service/update2"), brandcode_);
 }
 
 void ResetSettingsHandler::OnHideResetProfileDialog(
@@ -218,41 +213,17 @@ void ResetSettingsHandler::OnHideResetProfileBanner(
   chrome_prefs::ClearResetTime(profile_);
 }
 
-void ResetSettingsHandler::OnSettingsFetched() {
-  DCHECK(config_fetcher_);
-  DCHECK(!config_fetcher_->IsActive());
-  // The initial prefs is fetched. We are waiting for user pressing 'Reset'.
-}
-
 void ResetSettingsHandler::ResetProfile(
     const std::string& callback_id,
     bool send_settings,
     reset_report::ChromeResetReport::ResetRequestOrigin request_origin) {
-  CHECK(!GetResetter()->IsActive());
-
-  std::unique_ptr<BrandcodedDefaultSettings> default_settings;
-  if (config_fetcher_) {
-    DCHECK(!config_fetcher_->IsActive());
-    default_settings = config_fetcher_->GetSettings();
-    config_fetcher_.reset();
-  } else {
-    DCHECK(brandcode_.empty());
-  }
-
-  // If failed to fetch BrandcodedDefaultSettings or this is an organic
-  // installation, use default settings.
-  if (!default_settings)
-    default_settings = std::make_unique<BrandcodedDefaultSettings>();
-
-  GetResetter()->Reset(
-      ProfileResetter::ALL, std::move(default_settings),
+  GetResetter()->ResetSettings(
+      ProfileResetter::PROFILE_RESETS, nullptr,
       base::BindOnce(&ResetSettingsHandler::OnResetProfileSettingsDone,
                      callback_weak_ptr_factory_.GetWeakPtr(), callback_id,
                      send_settings, request_origin));
+
   base::RecordAction(base::UserMetricsAction("ResetProfile"));
-  UMA_HISTOGRAM_ENUMERATION(
-      "ProfileReset.ResetRequestOrigin", request_origin,
-      reset_report::ChromeResetReport::ResetRequestOrigin_MAX + 1);
 }
 
 ProfileResetter* ResetSettingsHandler::GetResetter() {
@@ -294,12 +265,39 @@ void ResetSettingsHandler::HandleGetTriggeredResetToolName(
 }
 
 #if BUILDFLAG(IS_CHROMEOS_ASH)
-void ResetSettingsHandler::OnShowPowerwashDialog(
-    const base::Value::List& args) {
-  UMA_HISTOGRAM_ENUMERATION(
-      "Reset.ChromeOS.PowerwashDialogShown",
-      ash::reset::DialogViewType::kFromOptions,
-      ash::reset::DialogViewType::kCount);
+void ResetSettingsHandler::OnShowSanitizeDialog(const base::Value::List& args) {
+  // TODO(b/357057195) move sanitize functionality functions out of
+  // ResetSettingsHandler and only leave the UI parts for ResetSettingsHandler.
+  if (base::FeatureList::IsEnabled(ash::features::kSanitize)) {
+    ash::SystemAppLaunchParams params;
+    params.launch_source = apps::LaunchSource::kUnknown;
+    ash::LaunchSystemWebAppAsync(ProfileManager::GetPrimaryUserProfile(),
+                                 ash::SystemWebAppType::OS_SANITIZE, params);
+  }
+}
+
+void ResetSettingsHandler::SanitizeSettings(const base::Value::List& args) {
+  ProfileResetter::ResettableFlags to_sanitize =
+      ProfileResetter::DEFAULT_SEARCH_ENGINE | ProfileResetter::HOMEPAGE |
+      ProfileResetter::CONTENT_SETTINGS | ProfileResetter::EXTENSIONS |
+      ProfileResetter::STARTUP_PAGES | ProfileResetter::PINNED_TABS |
+      ProfileResetter::SHORTCUTS | ProfileResetter::NTP_CUSTOMIZATIONS |
+      ProfileResetter::LANGUAGES | ProfileResetter::DNS_CONFIGURATIONS;
+  // TODO(b/319446147): get send_feedback flag and pass it down
+  GetResetter()->ResetSettings(
+      to_sanitize, nullptr,
+      base::BindOnce(&ResetSettingsHandler::OnSanitizeDone,
+                     callback_weak_ptr_factory_.GetWeakPtr()));
+
+  base::RecordAction(base::UserMetricsAction("Sanitize"));
+}
+
+void ResetSettingsHandler::OnSanitizeDone() {
+  setting_snapshot_.reset();
+  PrefService* prefs = ProfileManager::GetPrimaryUserProfile()->GetPrefs();
+  prefs->SetBoolean(ash::settings::prefs::kSanitizeCompleted, true);
+  prefs->CommitPendingWrite();
+  chrome::AttemptRestart();
 }
 #endif  // BUILDFLAG(IS_CHROMEOS_ASH)
 
